@@ -1,11 +1,15 @@
 import json
-import logging
+from logging import debug
+from cli_ui import debug as verbose
 
-import cli_ui
+import pkg_resources
 import requests
 import os
 
+# noinspection PyPackageRequirements
 import urllib3
+
+# noinspection PyPackageRequirements
 from urllib3.util.retry import Retry
 from urllib import parse
 from requests.adapters import HTTPAdapter
@@ -36,9 +40,19 @@ class GitLabCore:
         if not self.ssl_verify:
             urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
+        self.gitlabform_version = pkg_resources.get_distribution("gitlabform").version
+        self.requests_version = pkg_resources.get_distribution("requests").version
+        self.session.headers.update(
+            {
+                "private-token": self.token,
+                "authorization": f"Bearer {self.token}",
+                "user-agent": f"GitLabForm/{self.gitlabform_version} (python-requests/{self.requests_version})",
+            }
+        )
+
         try:
             version = self._make_requests_to_api("version")
-            cli_ui.debug(
+            verbose(
                 f"Connected to GitLab version: {version['version']} ({version['revision']})"
             )
             self.version = version["version"]
@@ -77,6 +91,10 @@ class GitLabCore:
         result = self.get_project(project_and_group)
         return str(result["id"])
 
+    def has_no_license(self):
+        license = self._make_requests_to_api("license")
+        return not license or license["expired"]
+
     def _make_requests_to_api(
         self,
         path_as_format_string,
@@ -164,7 +182,7 @@ class GitLabCore:
 
         if dict_data and json_data:
             raise Exception(
-                "You need to pass the either as dict (dict_data) or JSON (json_data), not both!"
+                "You need to pass the data either as dict (dict_data) or JSON (json_data), not both!"
             )
 
         url = (
@@ -172,55 +190,46 @@ class GitLabCore:
             + "/api/v4/"
             + self._format_with_url_encoding(path_as_format_string, args)
         )
-        logging.debug(
+        debug(
             "url = %s , method = %s , data = %s, json = %s",
             url,
             method,
             json.dumps(dict_data, sort_keys=True),
             json.dumps(json_data, sort_keys=True),
         )
-        headers = {
-            "PRIVATE-TOKEN": self.token,
-            "Authorization": "Bearer " + self.token,
-        }
         if dict_data:
             response = self.session.request(
-                method, url, headers=headers, data=dict_data, timeout=self.timeout
+                method, url, data=dict_data, timeout=self.timeout
             )
         elif json_data:
             response = self.session.request(
-                method, url, headers=headers, json=json_data, timeout=self.timeout
+                method, url, json=json_data, timeout=self.timeout
             )
         else:
-            response = self.session.request(
-                method, url, headers=headers, timeout=self.timeout
-            )
-        logging.debug("response code=%s" % response.status_code)
+            response = self.session.request(method, url, timeout=self.timeout)
+        debug("response code=%s" % response.status_code)
 
         if response.status_code in expected_codes:
             # if we accept error responses then they will likely not contain a JSON body
             # so fake it to fix further calls to response.json()
             if response.status_code == 204 or (400 <= response.status_code <= 499):
-                logging.debug("faking response body to be {}")
+                debug("faking response body to be {}")
                 response.json = lambda: {}
         else:
             if response.status_code == 404:
-                raise NotFoundException("Resource path='%s' not found!" % url)
+                raise NotFoundException(
+                    f"Resource with url='{url}' not found (HTTP 404)!"
+                )
             else:
+                data_output = f"data={dict_data}" if dict_data else f"json={json_data}"
                 raise UnexpectedResponseException(
-                    "Request url='%s', method=%s, data='%s' failed - expected code(s) %s, got code %s & body: '%s'"
-                    % (
-                        url,
-                        method,
-                        dict_data,
-                        str(expected_codes),
-                        response.status_code,
-                        response.content,
-                    ),
+                    f"Request url='{url}', method={method}, {data_output} failed -"
+                    f" expected code(s) {str(expected_codes)},"
+                    f" got code {response.status_code} & body: '{response.content}'",
                     response.status_code,
                 )
 
-        logging.debug("response json=%s" % json.dumps(response.json(), sort_keys=True))
+        debug("response json=%s" % json.dumps(response.json(), sort_keys=True))
         return response
 
     @staticmethod
@@ -240,7 +249,7 @@ class GitLabCore:
                     url_encoded_args += (parse.quote_plus(str(arg)),)
             else:
                 # URL-encode single arg
-                url_encoded_args = parse.quote_plus(single_arg_or_args_tuple)
+                url_encoded_args = parse.quote_plus(str(single_arg_or_args_tuple))
 
             return format_string % url_encoded_args
 
@@ -253,11 +262,8 @@ class GitLabCore:
 
 
 class TestRequestFailedException(Exception):
-    pass
-
-
-class ApiVersionIncorrectException(Exception):
-    pass
+    def __init__(self, underlying: Exception):
+        self.underlying = underlying
 
 
 class NotFoundException(Exception):
@@ -268,6 +274,10 @@ class TimeoutWaitingForDeletion(Exception):
     pass
 
 
+class InvalidParametersException(Exception):
+    pass
+
+
 class UnexpectedResponseException(Exception):
     def __init__(self, message, status_code):
         self.message = message
